@@ -1,3 +1,4 @@
+{-# LANGUAGE NoImplicitPrelude #-}
 -- |
 -- Module:      Data.Aeson
 -- Copyright:   (c) 2011-2016 Bryan O'Sullivan
@@ -113,7 +114,6 @@ module Data.Aeson
     , withObject
     , withText
     , withArray
-    , withNumber
     , withScientific
     , withBool
     , withEmbeddedJSON
@@ -131,7 +131,6 @@ module Data.Aeson
     , json'
     ) where
 
-import Prelude ()
 import Prelude.Compat
 
 import Data.Aeson.Types.FromJSON (ifromJSON)
