@@ -7,6 +7,7 @@
 {-# LANGUAGE GADTs #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
 {-# LANGUAGE NamedFieldPuns #-}
+{-# LANGUAGE NoImplicitPrelude #-}
 {-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE PatternGuards #-}
 {-# LANGUAGE RecordWildCards #-}
@@ -58,7 +59,6 @@ module Data.Aeson.Types.ToJSON
     , listValue
     ) where
 
-import Prelude ()
 import Prelude.Compat
 
 import Control.Applicative (Const(..))
@@ -66,7 +66,7 @@ import Control.Monad.ST (ST)
 import Data.Aeson.Encoding (Encoding, Encoding', Series, dict, emptyArray_)
 import Data.Aeson.Encoding.Internal ((>*<))
 import Data.Aeson.Internal.Functions (mapHashKeyVal, mapKeyVal)
-import Data.Aeson.Types.Generic (AllNullary, False, IsRecord(..), One, ProductSize, Tagged2(..), True, Zero, productSize)
+import Data.Aeson.Types.Generic (AllNullary, False, IsRecord, One, ProductSize, Tagged2(..), True, Zero, productSize)
 import Data.Aeson.Types.Internal
 import Data.Attoparsec.Number (Number(..))
 import Data.Bits (unsafeShiftR)
@@ -74,6 +74,7 @@ import Data.DList (DList)
 import Data.Fixed (Fixed, HasResolution)
 import Data.Foldable (toList)
 import Data.Functor.Compose (Compose(..))
+import Data.Functor.Contravariant (Contravariant (..))
 import Data.Functor.Identity (Identity(..))
 import Data.Functor.Product (Product(..))
 import Data.Functor.Sum (Sum(..))
@@ -91,6 +92,7 @@ import Data.Time.Format (FormatTime, formatTime)
 import Data.Time.Locale.Compat (defaultTimeLocale)
 import Data.Vector (Vector)
 import Data.Version (Version, showVersion)
+import Data.Void (Void, absurd)
 import Data.Word (Word16, Word32, Word64, Word8)
 import Foreign.Storable (Storable)
 import Foreign.C.Types (CTime (..))
@@ -127,6 +129,16 @@ import qualified Data.Vector.Unboxed as VU
 import qualified Data.Aeson.Encoding.Builder as EB
 import qualified Data.ByteString.Builder as B
 
+import qualified GHC.Exts as Exts
+import qualified Data.Primitive.Array as PM
+import qualified Data.Primitive.SmallArray as PM
+import qualified Data.Primitive.Types as PM
+
+#if MIN_VERSION_primitive(0,6,4)
+import qualified Data.Primitive.UnliftedArray as PM
+import qualified Data.Primitive.PrimArray as PM
+#endif
+
 #if !(MIN_VERSION_bytestring(0,10,0))
 import Foreign.ForeignPtr (withForeignPtr)
 import Foreign.Marshal.Utils (copyBytes)
@@ -264,7 +276,7 @@ genericLiftToEncoding opts te tel = gToJSON opts (To1Args te tel) . from1
 -- @
 --
 -- Previous versions of this library only had the 'toJSON' method. Adding
--- 'toEncoding' had to reasons:
+-- 'toEncoding' had two reasons:
 --
 -- 1. toEncoding is more efficient for the common case that the output of
 -- 'toJSON' is directly serialized to a @ByteString@.
@@ -335,6 +347,13 @@ instance KeyValue Pair where
     name .= value = (name, toJSON value)
     {-# INLINE (.=) #-}
 
+-- | Constructs a singleton 'H.HashMap'. For calling functions that
+--   demand an 'Object' for constructing objects. To be used in
+--   conjunction with 'mconcat'. Prefer to use 'object' where possible.
+instance KeyValue Object where
+    name .= value = H.singleton name (toJSON value)
+    {-# INLINE (.=) #-}
+
 -------------------------------------------------------------------------------
 --  Classes and types for map keys
 -------------------------------------------------------------------------------
@@ -487,6 +506,9 @@ toJSONKeyTextEnc e = ToJSONKeyText tot e
         . E.encodingToLazyByteString
         . e
 
+instance Contravariant ToJSONKeyFunction where
+    contramap = contramapToJSONKeyFunction
+
 -- | Contravariant map, as 'ToJSONKeyFunction' is a contravariant functor.
 contramapToJSONKeyFunction :: (b -> a) -> ToJSONKeyFunction a -> ToJSONKeyFunction b
 contramapToJSONKeyFunction h x = case x of
@@ -657,11 +679,13 @@ instance (ToJSON a) => ToJSON [a] where
 instance OVERLAPPABLE_ (GToJSON enc arity a) => GToJSON enc arity (M1 i c a) where
     -- Meta-information, which is not handled elsewhere, is ignored:
     gToJSON opts targs = gToJSON opts targs . unM1
+    {-# INLINE gToJSON #-}
 
 instance GToJSON enc One Par1 where
     -- Direct occurrences of the last type parameter are encoded with the
     -- function passed in as an argument:
     gToJSON _opts (To1Args tj _) = tj . unPar1
+    {-# INLINE gToJSON #-}
 
 instance ( ConsToJSON enc arity a
          , AllNullary          (C1 c a) allNullary
@@ -674,11 +698,13 @@ instance ( ConsToJSON enc arity a
                                      . sumToJSON opts targs
                                      . unM1
         | otherwise = consToJSON opts targs . unM1 . unM1
+    {-# INLINE gToJSON #-}
 
 instance (ConsToJSON enc arity a) => GToJSON enc arity (C1 c a) where
     -- Constructors need to be encoded differently depending on whether they're
     -- a record or not. This distinction is made by 'consToJSON':
     gToJSON opts targs = consToJSON opts targs . unM1
+    {-# INLINE gToJSON #-}
 
 instance ( AllNullary       (a :+: b) allNullary
          , SumToJSON  enc arity (a :+: b) allNullary
@@ -689,6 +715,7 @@ instance ( AllNullary       (a :+: b) allNullary
     -- strings.  This distinction is made by 'sumToJSON':
     gToJSON opts targs = (unTagged :: Tagged allNullary enc -> enc)
                        . sumToJSON opts targs
+    {-# INLINE gToJSON #-}
 
 --------------------------------------------------------------------------------
 -- Generic toJSON
@@ -697,18 +724,27 @@ instance ( AllNullary       (a :+: b) allNullary
 -- possible but makes error messages a bit harder to understand for missing
 -- instances.
 
+instance GToJSON Value arity V1 where
+    -- Empty values do not exist, which makes the job of formatting them
+    -- rather easy:
+    gToJSON _ _ x = x `seq` error "case: V1"
+    {-# INLINE gToJSON #-}
+
 instance ToJSON a => GToJSON Value arity (K1 i a) where
     -- Constant values are encoded using their ToJSON instance:
     gToJSON _opts _ = toJSON . unK1
+    {-# INLINE gToJSON #-}
 
 instance ToJSON1 f => GToJSON Value One (Rec1 f) where
     -- Recursive occurrences of the last type parameter are encoded using their
     -- ToJSON1 instance:
     gToJSON _opts (To1Args tj tjl) = liftToJSON tj tjl . unRec1
+    {-# INLINE gToJSON #-}
 
 instance GToJSON Value arity U1 where
     -- Empty constructors are encoded to an empty array:
     gToJSON _opts _ _ = emptyArray
+    {-# INLINE gToJSON #-}
 
 instance ( WriteProduct arity a, WriteProduct arity b
          , ProductSize        a, ProductSize        b
@@ -725,6 +761,7 @@ instance ( WriteProduct arity a, WriteProduct arity b
         where
           lenProduct = (unTagged2 :: Tagged2 (a :*: b) Int -> Int)
                        productSize
+    {-# INLINE gToJSON #-}
 
 instance ( ToJSON1 f
          , GToJSON Value One g
@@ -736,6 +773,7 @@ instance ( ToJSON1 f
     gToJSON opts targs =
       let gtj = gToJSON opts targs in
       liftToJSON gtj (listValue gtj) . unComp1
+    {-# INLINE gToJSON #-}
 
 --------------------------------------------------------------------------------
 -- Generic toEncoding
@@ -743,15 +781,18 @@ instance ( ToJSON1 f
 instance ToJSON a => GToJSON Encoding arity (K1 i a) where
     -- Constant values are encoded using their ToJSON instance:
     gToJSON _opts _ = toEncoding . unK1
+    {-# INLINE gToJSON #-}
 
 instance ToJSON1 f => GToJSON Encoding One (Rec1 f) where
     -- Recursive occurrences of the last type parameter are encoded using their
     -- ToEncoding1 instance:
     gToJSON _opts (To1Args te tel) = liftToEncoding te tel . unRec1
+    {-# INLINE gToJSON #-}
 
 instance GToJSON Encoding arity U1 where
     -- Empty constructors are encoded to an empty array:
     gToJSON _opts _ _ = E.emptyArray_
+    {-# INLINE gToJSON #-}
 
 instance ( EncodeProduct  arity a
          , EncodeProduct  arity b
@@ -761,6 +802,7 @@ instance ( EncodeProduct  arity a
     -- the same size as the product and write the product's elements to it using
     -- 'encodeProduct':
     gToJSON opts targs p = E.list E.retagEncoding [encodeProduct opts targs p]
+    {-# INLINE gToJSON #-}
 
 instance ( ToJSON1 f
          , GToJSON Encoding One g
@@ -772,6 +814,7 @@ instance ( ToJSON1 f
     gToJSON opts targs =
       let gte = gToJSON opts targs in
       liftToEncoding gte (listEncoding gte) . unComp1
+    {-# INLINE gToJSON #-}
 
 --------------------------------------------------------------------------------
 
@@ -983,6 +1026,7 @@ instance ( RecordToPairs enc pairs arity f
          ) => ConsToJSON' enc arity f True
   where
     consToJSON' opts targs = Tagged . fromPairs . recordToPairs opts targs
+    {-# INLINE consToJSON' #-}
 
 instance GToJSON enc arity f => ConsToJSON' enc arity f False where
     consToJSON' opts targs = Tagged . gToJSON opts targs
@@ -1074,10 +1118,12 @@ instance ( WriteProduct arity a
           lenL = len `unsafeShiftR` 1
           lenR = len - lenL
           ixR  = ix  + lenL
+    {-# INLINE writeProduct #-}
 
 instance OVERLAPPABLE_ (GToJSON Value arity a) => WriteProduct arity a where
     writeProduct opts targs mv ix _ =
       VM.unsafeWrite mv ix . gToJSON opts targs
+    {-# INLINE writeProduct #-}
 
 --------------------------------------------------------------------------------
 
@@ -1095,9 +1141,11 @@ instance ( EncodeProduct    arity a
     encodeProduct opts targs (a :*: b) =
       encodeProduct opts targs a >*<
       encodeProduct opts targs b
+    {-# INLINE encodeProduct #-}
 
 instance OVERLAPPABLE_ (GToJSON Encoding arity a) => EncodeProduct arity a where
     encodeProduct opts targs a = E.retagEncoding $ gToJSON opts targs a
+    {-# INLINE encodeProduct #-}
 
 --------------------------------------------------------------------------------
 
@@ -1201,6 +1249,13 @@ instance (ToJSON a, ToJSON b) => ToJSON (Either a b) where
     toEncoding = toEncoding2
     {-# INLINE toEncoding #-}
 
+instance ToJSON Void where
+    toJSON = absurd
+    {-# INLINE toJSON #-}
+
+    toEncoding = absurd
+    {-# INLINE toEncoding #-}
+
 
 instance ToJSON Bool where
     toJSON = Bool
@@ -1884,6 +1939,32 @@ dotNetTime (DotNetTime t) = secs ++ formatMillis t ++ ")/"
 formatMillis :: (FormatTime t) => t -> String
 formatMillis = take 3 . formatTime defaultTimeLocale "%q"
 
+-------------------------------------------------------------------------------
+-- primitive
+-------------------------------------------------------------------------------
+
+#if MIN_VERSION_base(4,7,0)
+instance ToJSON a => ToJSON (PM.Array a) where
+  -- note: we could do better than this if vector exposed the data
+  -- constructor in Data.Vector.
+  toJSON = toJSON . Exts.toList
+  toEncoding = toEncoding . Exts.toList
+
+instance ToJSON a => ToJSON (PM.SmallArray a) where
+  toJSON = toJSON . Exts.toList
+  toEncoding = toEncoding . Exts.toList
+
+#if (MIN_VERSION_primitive(0,6,4))
+instance (PM.Prim a,ToJSON a) => ToJSON (PM.PrimArray a) where
+  toJSON = toJSON . Exts.toList
+  toEncoding = toEncoding . Exts.toList
+
+instance (PM.PrimUnlifted a,ToJSON a) => ToJSON (PM.UnliftedArray a) where
+  toJSON = toJSON . Exts.toList
+  toEncoding = toEncoding . Exts.toList
+#endif
+#endif
+
 -------------------------------------------------------------------------------
 -- time
 -------------------------------------------------------------------------------
