@@ -1,7 +1,8 @@
 -- Manually write instances.
-
+{-# LANGUAGE NoImplicitPrelude #-}
 {-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE RecordWildCards #-}
+
 {-# OPTIONS_GHC -fno-warn-orphans #-}
 
 module Twitter.Manual
@@ -12,10 +13,10 @@ module Twitter.Manual
     , Result(..)
     ) where
 
-import Prelude ()
 import Prelude.Compat
 
 import Control.Applicative
+import Data.Semigroup ((<>))
 import Twitter
 
 import Data.Aeson hiding (Result)
